use strict;
use warnings;

package Test::A;

# FILENAME: A.pm
# CREATED: 26/03/12 14:03:00 by Kent Fredric (kentnl) <kentfredric@gmail.com>
# ABSTRACT: test package

our $VERSION = 1.00;

sub example {
  return "A(B)";
}

1;

